@@ -36,14 +36,6 @@ __func_alias__ = {
 log = logging.getLogger(__name__)
 
 
-def __virtual__():
-    commands = ('tar', 'gzip', 'gunzip', 'zip', 'unzip', 'rar', 'unrar')
-    # If none of the above commands are in $PATH this module is a no-go
-    if not any(salt.utils.which(cmd) for cmd in commands):
-        return (False, 'Unable to find commands tar,gzip,gunzip,zip,unzip,rar,unrar')
-    return True
-
-
 def list_(name,
           archive_format=None,
           options=None,
@@ -57,8 +49,20 @@ def list_(name,
 
     .. note::
         This function will only provide results for XZ-compressed archives if
-        xz-utils_ is installed, as Python does not at this time natively
-        support XZ compression in its tarfile_ module.
+        the xz_ CLI command is available, as Python does not at this time
+        natively support XZ compression in its tarfile_ module. Keep in mind
+        however that most Linux distros ship with xz_ already installed.
+
+        To check if a given minion has xz_, the following Salt command can be
+        run:
+
+        .. code-block:: bash
+
+            salt minion_id cmd.which xz
+
+        If ``None`` is returned, then xz_ is not present and must be installed.
+        It is widely available and should be packaged as either ``xz`` or
+        ``xz-utils``.
 
     name
         Path/URL of archive
@@ -108,7 +112,7 @@ def list_(name,
         the ``salt://`` fileserver.
 
     .. _tarfile: https://docs.python.org/2/library/tarfile.html
-    .. _xz-utils: http://tukaani.org/xz/
+    .. _xz: http://tukaani.org/xz/
 
     CLI Examples:
 
@@ -161,7 +165,7 @@ def list_(name,
 
         raise CommandExecutionError(
             'Unable to list contents of {0}. If this is an XZ-compressed tar '
-            'archive, install xz-utils to enable listing its contents. If it '
+            'archive, install XZ Utils to enable listing its contents. If it '
             'is compressed using something other than XZ, it may be necessary '
             'to specify CLI options to decompress the archive. See the '
             'documentation for details.'.format(name)
@@ -177,6 +181,10 @@ def list_(name,
             raise CommandExecutionError('{0} is not a ZIP file'.format(name))
 
     def _list_rar(name, cached):
+        if not salt.utils.which('rar'):
+            raise CommandExecutionError(
+                'rar command not available, is it installed?'
+            )
         output = __salt__['cmd.run'](
             ['rar', 'lt', path],
             python_shell=False,
@@ -185,7 +193,7 @@ def list_(name,
         ret = [x + '/' if y == 'Directory' else x for x, y in matches]
         if not ret:
             raise CommandExecutionError(
-                'Failed to decompress {0}'.format(name),
+                'Failed to list {0}, is it a rar file?'.format(name),
                 info={'error': output}
             )
         return ret
