@@ -490,7 +490,7 @@ def _netstat_route_sunos():
             'gateway': comps[1],
             'netmask': '',
             'flags': comps[2],
-            'interface': comps[5]})
+            'interface': comps[5] if len(comps) >= 6 else ''})
     cmd = 'netstat -f inet6 -rn | tail -n+5'
     out = __salt__['cmd.run'](cmd, python_shell=True)
     for line in out.splitlines():
@@ -501,7 +501,7 @@ def _netstat_route_sunos():
             'gateway': comps[1],
             'netmask': '',
             'flags': comps[2],
-            'interface': comps[5]})
+            'interface': comps[5] if len(comps) >= 6 else ''})
     return ret
 
 
@@ -1473,6 +1473,8 @@ def ifacestartswith(cidr):
     '''
     Retrieve the interface name from a specific CIDR
 
+    .. versionadded:: 2016.11.0
+
     CLI Example:
 
     .. code-block:: bash
@@ -1495,6 +1497,8 @@ def iphexval(ip):
     '''
     Retrieve the interface name from a specific CIDR
 
+    .. versionadded:: 2016.11.0
+
     CLI Example:
 
     .. code-block:: bash
