@@ -22,6 +22,7 @@ try:
     import win32api
     import win32con
     import pywintypes
+    import ctypes
     from ctypes import windll
     HAS_WIN32NET_MODS = True
 except ImportError:
@@ -30,7 +31,7 @@ except ImportError:
 # Import salt libs
 import salt.utils
 import salt.utils.locales
-from salt.modules.reg import read_value
+import salt.ext.six as six
 
 # Set up logging
 log = logging.getLogger(__name__)
@@ -55,6 +56,15 @@ def _convert_minutes_seconds(timeout, in_seconds=False):
     return timeout if in_seconds else timeout*60
 
 
+def _convert_date_time_string(dt_string):
+    '''
+    convert string to date time object
+    '''
+    dt_string = dt_string.split('.')[0]
+    dt_obj = datetime.strptime(dt_string, '%Y%m%d%H%M%S')
+    return dt_obj.strftime('%Y-%m-%d %H:%M:%S')
+
+
 def halt(timeout=5, in_seconds=False):
     '''
     Halt a running system.
@@ -83,7 +93,7 @@ def halt(timeout=5, in_seconds=False):
     return shutdown(timeout=timeout, in_seconds=in_seconds)
 
 
-def init(runlevel):
+def init(runlevel):  # pylint: disable=unused-argument
     '''
     Change the system runlevel on sysV compatible systems
 
@@ -93,11 +103,11 @@ def init(runlevel):
 
         salt '*' system.init 3
     '''
-    #cmd = ['init', runlevel]
-    #ret = __salt__['cmd.run'](cmd, python_shell=False)
-    #return ret
+    # cmd = ['init', runlevel]
+    # ret = __salt__['cmd.run'](cmd, python_shell=False)
+    # return ret
 
-    # TODO: Create a mapping of runlevels to
+    # TODO: Create a mapping of runlevels to  # pylint: disable=fixme
     #       corresponding Windows actions
 
     return 'Not implemented on Windows at this time.'
@@ -131,7 +141,8 @@ def poweroff(timeout=5, in_seconds=False):
     return shutdown(timeout=timeout, in_seconds=in_seconds)
 
 
-def reboot(timeout=5, in_seconds=False, wait_for_reboot=False):
+def reboot(timeout=5, in_seconds=False, wait_for_reboot=False,  # pylint: disable=redefined-outer-name
+           only_on_pending_reboot=False):
     '''
     Reboot a running system.
 
@@ -154,6 +165,14 @@ def reboot(timeout=5, in_seconds=False, wait_for_reboot=False):
 
         .. versionadded:: 2015.8.0
 
+    :param bool only_on_pending_reboot:
+
+        If this is set to True, then then the reboot will only proceed
+        if the system reports a pending reboot. Setting this paramater to
+        True could be useful when calling this function from a final housekeeping
+        state intended to be executed
+        at the end of a state run (using *order: last*).
+
     :return: True if successful
     :rtype: bool
 
@@ -163,9 +182,24 @@ def reboot(timeout=5, in_seconds=False, wait_for_reboot=False):
 
         salt '*' system.reboot 5
         salt '*' system.reboot 5 True
+
+    As example of invoking this function from within a final housekeeping state
+    is as follows:
+
+    Example:
+
+    .. code-block:: yaml
+
+        final housekeeping:
+           module.run:
+              - name: system.reboot
+              - only_on_pending_reboot: True
+              - order: last
+
     '''
 
-    ret = shutdown(timeout=timeout, reboot=True, in_seconds=in_seconds)
+    ret = shutdown(timeout=timeout, reboot=True, in_seconds=in_seconds,
+                   only_on_pending_reboot=only_on_pending_reboot)
 
     if wait_for_reboot:
         seconds = _convert_minutes_seconds(timeout, in_seconds)
@@ -174,7 +208,8 @@ def reboot(timeout=5, in_seconds=False, wait_for_reboot=False):
     return ret
 
 
-def shutdown(message=None, timeout=5, force_close=True, reboot=False, in_seconds=False):
+def shutdown(message=None, timeout=5, force_close=True, reboot=False,  # pylint: disable=redefined-outer-name
+             in_seconds=False, only_on_pending_reboot=False):
     '''
     Shutdown a running system.
 
@@ -212,6 +247,10 @@ def shutdown(message=None, timeout=5, force_close=True, reboot=False, in_seconds
         True restarts the computer immediately after shutdown.
         False caches to disk and safely powers down the system.
 
+    :param bool only_on_pending_reboot:
+        If this is set to True, then then shutdown will only proceed
+        if the system reports a pending reboot.
+
     :return: True if successful
     :rtype: bool
 
@@ -221,9 +260,12 @@ def shutdown(message=None, timeout=5, force_close=True, reboot=False, in_seconds
 
         salt '*' system.shutdown 5
     '''
-    seconds = _convert_minutes_seconds(timeout, in_seconds)
+    timeout = _convert_minutes_seconds(timeout, in_seconds)
 
-    if message:
+    if only_on_pending_reboot and not get_pending_reboot():
+        return True
+
+    if message and not isinstance(message, str):
         message = message.decode('utf-8')
     try:
         win32api.InitiateSystemShutdown('127.0.0.1', message, timeout,
@@ -261,6 +303,12 @@ def shutdown_abort():
 
     :return: True if successful
     :rtype: bool
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt 'minion-id' system.shutdown_abort
     '''
     try:
         win32api.AbortSystemShutdown('127.0.0.1')
@@ -280,6 +328,12 @@ def lock():
 
     :return: True if successful
     :rtype: bool
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt 'minion-id' system.lock
     '''
     return windll.user32.LockWorkStation()
 
@@ -301,7 +355,7 @@ def set_computer_name(name):
 
         salt 'minion-id' system.set_computer_name 'DavesComputer'
     '''
-    if name:
+    if name and six.PY2:
         name = name.decode('utf-8')
 
     if windll.kernel32.SetComputerNameExW(win32con.ComputerNamePhysicalDnsHostname,
@@ -333,9 +387,10 @@ def get_pending_computer_name():
         salt 'minion-id' system.get_pending_computer_name
     '''
     current = get_computer_name()
-    pending = read_value('HKLM',
-                         r'SYSTEM\CurrentControlSet\Services\Tcpip\Parameters',
-                         'NV Hostname')['vdata']
+    pending = __salt__['reg.read_value'](
+                'HKLM',
+                r'SYSTEM\CurrentControlSet\Services\Tcpip\Parameters',
+                'NV Hostname')['vdata']
     if pending:
         return pending if pending != current else None
     return False
@@ -378,10 +433,11 @@ def set_computer_desc(desc=None):
     system_info = win32net.NetServerGetInfo(None, 101)
 
     # If desc is passed, decode it for unicode
-    if desc:
-        system_info['comment'] = desc.decode('utf-8')
-    else:
+    if desc is None:
         return False
+    if not isinstance(desc, str):
+        desc = desc.decode('utf-8')
+    system_info['comment'] = desc
 
     # Apply new settings
     try:
@@ -397,7 +453,7 @@ def set_computer_desc(desc=None):
     return {'Computer Description': get_computer_desc()}
 
 
-set_computer_description = salt.utils.alias_function(set_computer_desc, 'set_computer_description')
+set_computer_description = salt.utils.alias_function(set_computer_desc, 'set_computer_description')  # pylint: disable=invalid-name
 
 
 def get_system_info():
@@ -408,15 +464,56 @@ def get_system_info():
         Returns a Dictionary containing information about the system to include
         name, description, version, etc...
     :rtype: dict
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt 'minion-id' system.get_info
     '''
-    system_info = win32net.NetServerGetInfo(None, 101)
-    return system_info
+    os_type = {1: 'Work Station',
+               2: 'Domain Controller',
+               3: 'Server'}
+    pythoncom.CoInitialize()
+    conn = wmi.WMI()
+    system = conn.Win32_OperatingSystem()[0]
+    ret = {'name': get_computer_name(),
+           'description': system.Description,
+           'install_date': system.InstallDate,
+           'last_boot': system.LastBootUpTime,
+           'os_manufacturer': system.Manufacturer,
+           'os_name': system.Caption,
+           'users': system.NumberOfUsers,
+           'organization': system.Organization,
+           'os_architecture': system.OSArchitecture,
+           'primary': system.Primary,
+           'os_type': os_type[system.ProductType],
+           'registered_user': system.RegisteredUser,
+           'system_directory': system.SystemDirectory,
+           'system_drive': system.SystemDrive,
+           'os_version': system.Version,
+           'windows_directory': system.WindowsDirectory}
+    system = conn.Win32_ComputerSystem()[0]
+    ret.update({'hardware_manufacturer': system.Manufacturer,
+                'hardware_model': system.Model,
+                'processors': system.NumberOfProcessors,
+                'processors_logical': system.NumberOfLogicalProcessors,
+                'system_type': system.SystemType})
+    system = conn.Win32_BIOS()[0]
+    ret.update({'hardware_serial': system.SerialNumber,
+                'bios_manufacturer': system.Manufacturer,
+                'bios_version': system.Version,
+                'bios_details': system.BIOSVersion,
+                'bios_caption': system.Caption,
+                'bios_description': system.Description})
+    ret['install_date'] = _convert_date_time_string(ret['install_date'])
+    ret['last_boot'] = _convert_date_time_string(ret['last_boot'])
+    return ret
 
 
 def get_computer_desc():
     '''
     Get the Windows computer description
-
     :return:
         Returns the computer description if found. Otherwise returns False
 
@@ -426,11 +523,11 @@ def get_computer_desc():
 
         salt 'minion-id' system.get_computer_desc
     '''
-    desc = get_system_info()['comment']
-    return desc if desc else False
+    desc = get_system_info()['description']
+    return False if desc is None else desc
 
 
-get_computer_description = salt.utils.alias_function(get_computer_desc, 'get_computer_description')
+get_computer_description = salt.utils.alias_function(get_computer_desc, 'get_computer_description')  # pylint: disable=invalid-name
 
 
 def get_hostname():
@@ -448,9 +545,9 @@ def get_hostname():
 
         salt 'minion-id' system.get_hostname
     '''
-    cmd = 'wmic computersystem get name'
+    cmd = 'wmic nicconfig get dnshostname'
     ret = __salt__['cmd.run'](cmd=cmd)
-    _, hostname = ret.split("\n")
+    _, _, hostname = ret.split("\n")
     return hostname
 
 
@@ -568,19 +665,21 @@ def join_domain(domain,
         account_ou = account_ou.split('\\')
         account_ou = ''.join(account_ou)
 
-    NETSETUP_JOIN_DOMAIN = 0x1
-    NETSETUP_ACCOUNT_CREATE = 0x2
-    NETSETUP_DOMAIN_JOIN_IF_JOINED = 0x20
+    NETSETUP_JOIN_DOMAIN = 0x1  # pylint: disable=invalid-name
+    NETSETUP_ACCOUNT_CREATE = 0x2  # pylint: disable=invalid-name
+    NETSETUP_DOMAIN_JOIN_IF_JOINED = 0x20  # pylint: disable=invalid-name
+    NETSETUP_JOIN_WITH_NEW_NAME = 0x400  # pylint: disable=invalid-name
 
     join_options = 0x0
     join_options |= NETSETUP_JOIN_DOMAIN
     join_options |= NETSETUP_DOMAIN_JOIN_IF_JOINED
+    join_options |= NETSETUP_JOIN_WITH_NEW_NAME
     if not account_exists:
         join_options |= NETSETUP_ACCOUNT_CREATE
 
     pythoncom.CoInitialize()
-    c = wmi.WMI()
-    comp = c.Win32_ComputerSystem()[0]
+    conn = wmi.WMI()
+    comp = conn.Win32_ComputerSystem()[0]
     err = comp.JoinDomainOrWorkgroup(Name=domain,
                                      Password=password,
                                      UserName=username,
@@ -660,15 +759,15 @@ def unjoin_domain(username=None,
     if username and password is None:
         return 'Must specify a password if you pass a username'
 
-    NETSETUP_ACCT_DELETE = 0x4  # pylint: disable=invalid-name
+    NETSETUP_ACCT_DELETE = 0x2  # pylint: disable=invalid-name
 
     unjoin_options = 0x0
     if disable:
         unjoin_options |= NETSETUP_ACCT_DELETE
 
     pythoncom.CoInitialize()
-    c = wmi.WMI()
-    comp = c.Win32_ComputerSystem()[0]
+    conn = wmi.WMI()
+    comp = conn.Win32_ComputerSystem()[0]
     err = comp.UnjoinDomainOrWorkgroup(Password=password,
                                        UserName=username,
                                        FUnjoinOptions=unjoin_options)
@@ -704,24 +803,66 @@ def get_domain_workgroup():
     :return: The name of the domain or workgroup
     :rtype: str
 
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt 'minion-id' system.get_domain_workgroup
+
     '''
     pythoncom.CoInitialize()
-    c = wmi.WMI()
-    for computer in c.Win32_ComputerSystem():
+    conn = wmi.WMI()
+    for computer in conn.Win32_ComputerSystem():
         if computer.PartOfDomain:
             return {'Domain': computer.Domain}
         else:
             return {'Workgroup': computer.Domain}
 
 
+def _try_parse_datetime(time_str, fmts):
+    '''
+    Attempts to parse the input time_str as a date.
+
+    :param str time_str: A string representing the time
+    :param list fmts: A list of date format strings
+
+    :return: Returns a datetime object if parsed properly. Otherwise None
+    :rtype datetime
+    '''
+    result = None
+    for fmt in fmts:
+        try:
+            result = datetime.strptime(time_str, fmt)
+            break
+        except ValueError:
+            pass
+    return result
+
+
 def get_system_time():
     '''
     Get the system time.
 
-    :return: Returns the system time in HH:MM AM/PM format.
+    :return: Returns the system time in HH:MM:SS AM/PM format.
     :rtype: str
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt 'minion-id' system.get_system_time
     '''
-    return datetime.strftime(datetime.now(), "%I:%M %p")
+    now = win32api.GetLocalTime()
+    meridian = 'AM'
+    hours = int(now[4])
+    if hours == 12:
+        meridian = 'PM'
+    elif hours == 0:
+        hours = 12
+    elif hours > 12:
+        hours = hours - 12
+        meridian = 'PM'
+    return '{0:02d}:{1:02d}:{2:02d} {3}'.format(hours, now[5], now[6], meridian)
 
 
 def set_system_time(newtime):
@@ -737,14 +878,23 @@ def set_system_time(newtime):
 
     :return: Returns True if successful. Otherwise False.
     :rtype: bool
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt 'minion-id' system.set_system_time 12:01
     '''
     # Get date/time object from newtime
-    dt_obj = salt.utils.date_cast(newtime)
+    fmts = ['%I:%M:%S %p', '%I:%M %p', '%H:%M:%S', '%H:%M']
+    dt_obj = _try_parse_datetime(newtime, fmts)
+    if dt_obj is None:
+        return False
 
     # Set time using set_system_date_time()
-    return set_system_date_time(hours=int(dt_obj.strftime('%H')),
-                                minutes=int(dt_obj.strftime('%M')),
-                                seconds=int(dt_obj.strftime('%S')))
+    return set_system_date_time(hours=dt_obj.hour,
+                                minutes=dt_obj.minute,
+                                seconds=dt_obj.second)
 
 
 def set_system_date_time(years=None,
@@ -800,17 +950,29 @@ def set_system_date_time(years=None,
     if seconds is None:
         seconds = date_time[6]
 
-    # Create the time tuple to be passed to SetLocalTime, including day_of_week
-    time_tuple = (years, months, days, hours, minutes, seconds, 0)
-
     try:
-        win32api.SetLocalTime(time_tuple)
-    except win32api.error as exc:
-        (number, context, message) = exc
+        class SYSTEMTIME(ctypes.Structure):
+            _fields_ = [
+                ('wYear', ctypes.c_int16),
+                ('wMonth', ctypes.c_int16),
+                ('wDayOfWeek', ctypes.c_int16),
+                ('wDay', ctypes.c_int16),
+                ('wHour', ctypes.c_int16),
+                ('wMinute', ctypes.c_int16),
+                ('wSecond', ctypes.c_int16),
+                ('wMilliseconds', ctypes.c_int16)]
+        system_time = SYSTEMTIME()
+        system_time.wYear = int(years)
+        system_time.wMonth = int(months)
+        system_time.wDay = int(days)
+        system_time.wHour = int(hours)
+        system_time.wMinute = int(minutes)
+        system_time.wSecond = int(seconds)
+        system_time_ptr = ctypes.pointer(system_time)
+        succeeded = ctypes.windll.kernel32.SetLocalTime(system_time_ptr)
+        return succeeded is not 0
+    except OSError:
         log.error('Failed to set local time')
-        log.error('nbr: {0}'.format(number))
-        log.error('ctx: {0}'.format(context))
-        log.error('msg: {0}'.format(message))
         return False
 
     return True
@@ -829,7 +991,8 @@ def get_system_date():
 
         salt '*' system.get_system_date
     '''
-    return datetime.strftime(datetime.now(), "%a %m/%d/%Y")
+    now = win32api.GetLocalTime()
+    return '{0:02d}/{1:02d}/{2:04d}'.format(now[1], now[3], now[0])
 
 
 def set_system_date(newdate):
@@ -851,13 +1014,17 @@ def set_system_date(newdate):
 
         salt '*' system.set_system_date '03-28-13'
     '''
+    fmts = ['%Y-%m-%d', '%m-%d-%Y', '%m-%d-%y',
+            '%m/%d/%Y', '%m/%d/%y', '%Y/%m/%d']
     # Get date/time object from newdate
-    dt_obj = salt.utils.date_cast(newdate)
+    dt_obj = _try_parse_datetime(newdate, fmts)
+    if dt_obj is None:
+        return False
 
     # Set time using set_system_date_time()
-    return set_system_date_time(years=int(dt_obj.strftime('%Y')),
-                                months=int(dt_obj.strftime('%m')),
-                                days=int(dt_obj.strftime('%d')))
+    return set_system_date_time(years=dt_obj.year,
+                                months=dt_obj.month,
+                                days=dt_obj.day)
 
 
 def start_time_service():
@@ -890,3 +1057,279 @@ def stop_time_service():
         salt '*' system.stop_time_service
     '''
     return __salt__['service.stop']('w32time')
+
+
+def get_pending_component_servicing():
+    '''
+    Determine whether there are pending Component Based Servicing tasks that require a reboot.
+
+    :return: A boolean representing whether there are pending Component Based Servicing tasks.
+    :rtype: bool
+
+    .. versionadded:: 2016.11.0
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' system.get_pending_component_servicing
+    '''
+    vname = '(Default)'
+    key = r'SOFTWARE\Microsoft\Windows\CurrentVersion\Component Based Servicing\RebootPending'
+
+    reg_ret = __salt__['reg.read_value']('HKLM', key, vname)
+
+    # So long as the registry key exists, a reboot is pending.
+    if reg_ret['success']:
+        log.debug('Found key: %s', key)
+        return True
+    else:
+        log.debug('Unable to access key: %s', key)
+    return False
+
+
+def get_pending_domain_join():
+    '''
+    Determine whether there is a pending domain join action that requires a reboot.
+
+    :return: A boolean representing whether there is a pending domain join action.
+    :rtype: bool
+
+    .. versionadded:: 2016.11.0
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' system.get_pending_domain_join
+    '''
+    vname = '(Default)'
+    base_key = r'SYSTEM\CurrentControlSet\Services\Netlogon'
+    avoid_key = r'{0}\AvoidSpnSet'.format(base_key)
+    join_key = r'{0}\JoinDomain'.format(base_key)
+
+    # If either the avoid_key or join_key is present,
+    # then there is a reboot pending.
+
+    avoid_reg_ret = __salt__['reg.read_value']('HKLM', avoid_key, vname)
+
+    if avoid_reg_ret['success']:
+        log.debug('Found key: %s', avoid_key)
+        return True
+    else:
+        log.debug('Unable to access key: %s', avoid_key)
+
+    join_reg_ret = __salt__['reg.read_value']('HKLM', join_key, vname)
+
+    if join_reg_ret['success']:
+        log.debug('Found key: %s', join_key)
+        return True
+    else:
+        log.debug('Unable to access key: %s', join_key)
+    return False
+
+
+def get_pending_file_rename():
+    '''
+    Determine whether there are pending file rename operations that require a reboot.
+
+    :return: A boolean representing whether there are pending file rename operations.
+    :rtype: bool
+
+    .. versionadded:: 2016.11.0
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' system.get_pending_file_rename
+    '''
+    vnames = ('PendingFileRenameOperations', 'PendingFileRenameOperations2')
+    key = r'SYSTEM\CurrentControlSet\Control\Session Manager'
+
+    # If any of the value names exist and have value data set,
+    # then a reboot is pending.
+
+    for vname in vnames:
+        reg_ret = __salt__['reg.read_value']('HKLM', key, vname)
+
+        if reg_ret['success']:
+            log.debug('Found key: %s', key)
+
+            if reg_ret['vdata'] and (reg_ret['vdata'] != '(value not set)'):
+                return True
+        else:
+            log.debug('Unable to access key: %s', key)
+    return False
+
+
+def get_pending_servermanager():
+    '''
+    Determine whether there are pending Server Manager tasks that require a reboot.
+
+    :return: A boolean representing whether there are pending Server Manager tasks.
+    :rtype: bool
+
+    .. versionadded:: 2016.11.0
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' system.get_pending_servermanager
+    '''
+    vname = 'CurrentRebootAttempts'
+    key = r'SOFTWARE\Microsoft\ServerManager'
+
+    # There are situations where it's possible to have '(value not set)' as
+    # the value data, and since an actual reboot wont be pending in that
+    # instance, just catch instances where we try unsuccessfully to cast as int.
+
+    reg_ret = __salt__['reg.read_value']('HKLM', key, vname)
+
+    if reg_ret['success']:
+        log.debug('Found key: %s', key)
+
+        try:
+            if int(reg_ret['vdata']) > 0:
+                return True
+        except ValueError:
+            pass
+    else:
+        log.debug('Unable to access key: %s', key)
+    return False
+
+
+def get_pending_update():
+    '''
+    Determine whether there are pending updates that require a reboot.
+
+    :return: A boolean representing whether there are pending updates.
+    :rtype: bool
+
+    .. versionadded:: 2016.11.0
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' system.get_pending_update
+    '''
+    vname = '(Default)'
+    key = r'SOFTWARE\Microsoft\Windows\CurrentVersion\WindowsUpdate\Auto Update\RebootRequired'
+
+    reg_ret = __salt__['reg.read_value']('HKLM', key, vname)
+
+    # So long as the registry key exists, a reboot is pending.
+    if reg_ret['success']:
+        log.debug('Found key: %s', key)
+        return True
+    else:
+        log.debug('Unable to access key: %s', key)
+    return False
+
+
+MINION_VOLATILE_KEY = r'SYSTEM\CurrentControlSet\Services\salt-minion\Volatile-Data'
+
+
+REBOOT_REQUIRED_NAME = 'Reboot required'
+
+
+def set_reboot_required_witnessed():
+    r'''
+    .. versionadded:: 2016.11.0
+
+    This function is used to remember that
+    an event indicating that a reboot is required was witnessed.
+    This function relies on the salt-minion's ability to create the following
+    volatile registry key in the *HKLM* hive:
+
+       *SYSTEM\\CurrentControlSet\\Services\\salt-minion\\Volatile-Data*
+
+    Because this registry key is volatile, it will not persist
+    beyond the current boot session.
+    Also, in the scope of this key, the name *'Reboot required'* will be
+    assigned the value of *1*.
+
+    (For the time being, this this function is being used
+    whenever an install completes with exit code 3010 and
+    this usage can be extended where appropriate in the future.)
+
+    :return: A boolean indicating whether or not the salt minion was
+       able to perform the necessary registry operations.
+
+    :rtype: bool
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' system.set_reboot_required_witnessed
+
+    '''
+    return __salt__['reg.set_value'](hive='HKLM',
+                                     key=MINION_VOLATILE_KEY,
+                                     volatile=True,
+                                     vname=REBOOT_REQUIRED_NAME,
+                                     vdata=1,
+                                     vtype='REG_DWORD')
+
+
+def get_reboot_required_witnessed():
+    '''
+    .. versionadded:: 2016.11.0
+
+    This tells us if, at any time during the current boot session
+    the salt minion witnessed an event indicating
+    that a reboot is required.
+    (For the time being, this function will return True
+    if an install completed with exit code 3010 during the current
+    boot session and this usage can be extended where appropriate
+    in the future)
+
+    :return: a boolean which will be True if the salt-minion reported
+       a required reboot during the current boot session, otherwise False.
+
+    :rtype: bool
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' system.get_reboot_required_witnessed
+
+    '''
+    value_dict = __salt__['reg.read_value'](hive='HKLM',
+                                            key=MINION_VOLATILE_KEY,
+                                            vname=REBOOT_REQUIRED_NAME)
+    return value_dict['vdata'] == 1
+
+
+def get_pending_reboot():
+    '''
+    Determine whether there is a reboot pending.
+
+    :return: A boolean representing whether reboots are pending.
+    :rtype: bool
+
+    .. versionadded:: 2016.11.0
+
+    CLI Example:
+
+    .. code-block:: bash
+
+        salt '*' system.get_pending_reboot
+    '''
+
+    # Order the checks for reboot pending in most to least likely.
+    checks = (get_pending_update, get_pending_file_rename, get_pending_servermanager,
+              get_pending_component_servicing,
+              get_reboot_required_witnessed,
+              get_pending_computer_name,
+              get_pending_domain_join)
+
+    for check in checks:
+        if check():
+            return True
+
+    return False
