@@ -187,6 +187,14 @@ class CkMinions(object):
         else:
             self.acc = 'accepted'
 
+    def _check_nodegroup_minions(self, expr, greedy):  # pylint: disable=unused-argument
+        '''
+        Return minions found by looking at nodegroups
+        '''
+        return self._check_compound_minions(nodegroup_comp(expr, self.opts['nodegroups']),
+            DEFAULT_TARGET_DELIM,
+            greedy)
+
     def _check_glob_minions(self, expr, greedy):  # pylint: disable=unused-argument
         '''
         Return the minions found by looking via globs
